import floppyforms as forms
from registration.forms import RegistrationForm
from models import *
from registration.models import RegistrationProfile
import json
from django.forms.models import inlineformset_factory
from django.contrib.contenttypes.generic import generic_inlineformset_factory
from padmavideos.models import PadmaVideo, PadmaClip
from crispy_forms.helper import FormHelper

class ItfForm(forms.ModelForm):
        
    def __init__(self, *args, **kwargs):
        self.helper = FormHelper()
        super(ItfForm, self).__init__(*args, **kwargs)


class ConnectionsWidget(forms.SelectMultiple):
    template_name = 'formwidgets/select2_multiple.html'

    '''
    def get_context_data(self):
        ctx = super(ConnectionsWidget, self).get_context_data()
        
        #ctx['data_json'] = [{'id': obj[0], 'text': obj[1]} for obj in self.choices]
        return ctx
    '''
        
    def get_context(self, name, value, attrs, *args, **kwargs):
        ctx = super(ConnectionsWidget, self).get_context(name, value, attrs)
        ctx['popup_url'] = '/m/person/popup' #FIXME: url scheme, also, a better way to pass this context to the widget template.
        #selected_values = [{'id': obj[0], 'text': obj[1]} for obj in self.choices if obj[0] in value]
        #ctx['data_json'] = json.dumps(selected_values) # json.dumps(choices)
        return ctx
    

class ItfRegistrationForm(RegistrationForm):
    subscribe = forms.BooleanField(help_text='Subscribe to newsletter?')

    def save(self, profile_callback=None):
        new_user = RegistrationProfile.objects.create_inactive_user(username=self.cleaned_data['username'], password=self.cleaned_data['password1'], email=self.cleaned_data['email'])
        new_profile = ItfProfile(user=new_user, subscribed=self.cleaned_data['subscribe'])
        new_profile.save()
        return new_user
    

class ConnectionsForm(ItfForm):
    person2 = forms.ModelMultipleChoiceField(Person.objects.all(), widget=ConnectionsWidget())
    
    class Meta:
        model = PersonPerson
        exclude = ('disapproved',)

class PersonProductionForm(ItfForm):
    pass

    class Meta:
        model = PersonProduction

class PersonGroupForm(ItfForm):
    pass

    class Meta:
        model = PersonGroup

class PadmaClipForm(ItfForm):
    pass

    class Meta:
        model = PadmaClip


def itf_inlineformset_factory(model, *args, **kwargs):
    title = kwargs.pop('title', '')
    help_text = kwargs.pop('help_text', '')
    is_generic = kwargs.pop('is_generic', False)
    #kwargs.delete("title")
    if is_generic:
        FormSet = generic_inlineformset_factory(model, *args, **kwargs)
    else:
        FormSet = inlineformset_factory(model, *args, **kwargs)   
    FormSet.title = title
    FormSet.help_text = help_text
    return FormSet

ConnectionsInline = itf_inlineformset_factory(Person, PersonPerson, fk_name='person1', form=ConnectionsForm, extra=1, title="Add / Edit Connections", help_text="select the people you are connected with and how you are related in the theatre world")
ProductionsInline = itf_inlineformset_factory(Person, PersonProduction, form=PersonProductionForm, extra=1, title="Productions you have worked in")
GroupsInline = itf_inlineformset_factory(Person, PersonGroup, extra=1, form=PersonGroupForm, title="Theatre groups you are a part of or have worked with")
PadmaClipsInline = itf_inlineformset_factory(PadmaClip, extra=1, is_generic=True, form=PadmaClipForm, title="Videos you appear in")


#Actual person form definition
class PersonForm(forms.ModelForm):
#    occupations = forms.ModelMultipleChoiceField(Occupation.objects.all(), widget=forms.CheckboxSelectMultiple())
#    connections = forms.ModelMultipleChoiceField(Person.objects.all(), widget=ConnectionsWidget())
#    occupations = forms.ModelMultipleChoiceField(Occupation.objects.all(), widget=forms.CheckboxSelectMultiple())
    email = forms.EmailField(widget=forms.EmailInput(attrs={'placeholder': 'john@example.com'}))
#    inlines = [ConnectionsInline]   
    inlines = [ConnectionsInline, ProductionsInline, GroupsInline, PadmaClipsInline]

    class Meta:
        exclude = ('connections', 'productions', 'groups', 'last_accessed',)
        model = Person
        widgets = {
            'first_name': forms.TextInput,
            'last_name': forms.TextInput,
            'occupations': forms.CheckboxSelectMultiple
        }
        #exclude = ('connections', 'productions')
      

